require 'test_helper'
require 'minitest/autorun'
require 'yaml_editor'
require 'pp'

# Unit Test
# rubocop:disable Metrics/ClassLength
class TestYamlEditor < Minitest::Test
  def setup
    testfile = File.expand_path('../../data/testdata.yml', __FILE__)
    @ed = YamlEditor.new(File.read(testfile))
  end

  def test_bracket_value
    _spos, _epos, kind, v = @ed.bracket('keya', 'second', 'KEY_A')
    assert_equal(:value, kind)
    _tag, value, comment = v
    assert_equal('valuea', value)
    assert_empty(comment)
  end

  def test_bracket_value_quote
    _spos, _epos, kind, v = @ed.bracket('keya', 'second', 'KEY_C')
    assert_equal(:value, kind)
    _tag, value, comment = v
    assert_equal('\'value c\'', value)
    assert_empty(comment)
  end

  def test_get_hash
    spos, epos, kind, = @ed.bracket('keya', 'second')
    assert_equal(:hash, kind)
    assert_equal(3, epos - spos) # 4 values but zero based lines
  end

  def test_get_array
    spos, epos, kind, = @ed.bracket('keyb')
    assert_equal(:array, kind)
    assert_equal(18, epos - spos)
  end

  def test_bracket_value_comment
    _spos, _epos, kind, v = @ed.bracket('keya', 'second', 'KEY_D')
    assert_equal(:value, kind)
    tag, value, comment = v
    assert_includes(tag, 'KEY_D:')
    assert_equal('"value d"', value)
    assert_equal(' # Comment', comment)
  end

  def test_array_index
    spos, epos, kind, = @ed.bracket('keyb')
    assert_equal(:array, kind)
    arr = @ed.array(spos, epos)
    assert_kind_of(Array, arr)
    assert(3, arr.first.last - arr.first.first)
    # Move out to another test after embedding array indexing in bracketing.
    assert_includes(@ed.lines(arr.last.first), 'standalone value')
  end

  def test_array_bracket
    _spos, _epos, kind, v = @ed.bracket('keyb', 1, 'KEY_B')
    assert_equal(:value, kind)
    tag, value, comment = v
    assert_includes(tag, 'KEY_B:')
    assert_equal('valuef', value)
    assert_equal(' # Comment', comment)
  end

  def test_collection_hashes_0
    _spos, _epos, _kind, v = @ed.bracket('keyc', 'coll', 0, 'name')
    _tag, value, = v
    assert_equal('\'index 0\'', value)
  end

  def test_collection_hashes_1
    _spos, _epos, _kind, v = @ed.bracket('keyc', 'coll', 1, 'name')
    _tag, value, = v
    assert_equal('\'index 1\'', value)
  end

  def test_update
    @ed.update('keyb', 1, 'KEY_D', val: 'updated')
    _spos, _epos, kind, v = @ed.bracket('keyb', 1, 'KEY_D')
    assert_equal(:value, kind)
    tag, value, comment = v
    assert_includes(tag, 'KEY_D:')
    assert_equal('updated', value)
    assert_equal(' # Comment', comment)
  end

  def test_spaced_update
    @ed.update('keyb', 1, 'KEY_D', val: '"updated with spaces"')
    _spos, _epos, kind, v = @ed.bracket('keyb', 1, 'KEY_D')
    assert_equal(:value, kind)
    tag, value, comment = v
    assert_includes(tag, 'KEY_D:')
    assert_equal('"updated with spaces"', value)
    assert_equal(' # Comment', comment)
  end

  def test_bad_quotes
    assert_raises(YamlEditorError) do
      @ed.update('keyb', 1, 'KEY_D', val: 'should not work')
    end
  end

  def test_update_append
    @ed.update('keyb', 1, 'KEY_E', val: 'appended')
    spos, epos, = @ed.bracket('keyb', 1)
    assert_equal(4, epos - spos)
    _spos, _epos, kind, v = @ed.inner_bracket('KEY_E', spos, epos)
    assert_equal(:value, kind)
    _tag, value, = v
    assert_equal('appended', value)
  end

  def test_to_yaml
    assert_equal(45, @ed.to_yaml.lines.count)
  end

  def test_hashes_in_collections_a
    @ed.update('keyd', 0, 'nested', 0, 'KEY_A', val: 'changed')
    _spos, _epos, _kind, v = @ed.bracket('keyd', 0, 'nested', 0, 'KEY_A')
    tag, value, = v
    assert_includes(tag, 'KEY_A:')
    assert_equal('changed', value)
  end

  def test_hashes_in_collections_b
    @ed.update('keyd', 0, 'nested', 0, 'KEY_C', val: 'appended')
    _spos, _epos, _kind, v = @ed.bracket('keyd', 0, 'nested', 0, 'KEY_C')
    tag, value, = v
    assert_includes(tag, 'KEY_C:')
    assert_equal('appended', value)
  end
end
